import { defaultStyles } from './codes.js';
import { convertColorSpec } from './spec.js';
import { customStyles } from './styles.js';
export * from './codes.js';
export * from './spec.js';
export * from './styles.js';
export var templateMatcher = /{{2}([^{}]+?)}{2}/gi;
export function colorize(raw) {
    // Create a new styles stack
    var stylesStack = [];
    var stylesInserted = false;
    // For each tag in the string
    var modified = raw.replace(templateMatcher, function(_, spec) {
        var revert = [];
        var replacement = '';
        // Get all the styles - Whatever is not found in ansi-styles is ignored
        var tokens = spec.trim().split(/\s+/).map(function(s) {
            return s.trim();
        });
        var _iteratorNormalCompletion = true, _didIteratorError = false, _iteratorError = undefined;
        try {
            for(var _iterator = tokens[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true){
                var token = _step.value;
                if (token === '-') {
                    // Remove a style
                    if (stylesStack.length) {
                        // First of all, remove the latest applied style
                        replacement += stylesStack.shift().reverse().map(function(s) {
                            return s.close;
                        }).join('');
                        // If there is a style to restore it, reapply it
                        if (stylesStack.length) {
                            replacement += stylesStack[0].map(function(s) {
                                return s.open;
                            }).join('');
                        }
                    }
                    break;
                } else if (token === 'reset') {
                    // Forget all previously defined styles
                    stylesStack = [];
                    break;
                } else {
                    var ref;
                    // Search in custom styles first
                    var styles = (ref = customStyles.get(token)) !== null && ref !== void 0 ? ref : [
                        token
                    ];
                    var _iteratorNormalCompletion1 = true, _didIteratorError1 = false, _iteratorError1 = undefined;
                    try {
                        for(var _iterator1 = styles[Symbol.iterator](), _step1; !(_iteratorNormalCompletion1 = (_step1 = _iterator1.next()).done); _iteratorNormalCompletion1 = true){
                            var style = _step1.value;
                            // Add a style
                            var code = convertColorSpec(style);
                            if (code) {
                                replacement += code.open;
                                revert.push(code);
                            }
                        }
                    } catch (err) {
                        _didIteratorError1 = true;
                        _iteratorError1 = err;
                    } finally{
                        try {
                            if (!_iteratorNormalCompletion1 && _iterator1.return != null) {
                                _iterator1.return();
                            }
                        } finally{
                            if (_didIteratorError1) {
                                throw _iteratorError1;
                            }
                        }
                    }
                }
            }
        } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
        } finally{
            try {
                if (!_iteratorNormalCompletion && _iterator.return != null) {
                    _iterator.return();
                }
            } finally{
                if (_didIteratorError) {
                    throw _iteratorError;
                }
            }
        }
        // If anything was applied, append to the stack
        if (revert.length) {
            stylesInserted = true;
            stylesStack.unshift(revert);
        }
        return replacement;
    });
    // Always make sure to reset codes at the end
    if (stylesInserted) {
        modified += defaultStyles.reset.close;
    }
    return modified;
}
export function clean(raw) {
    return raw.replace(templateMatcher, '');
}
